from bge import logic, render
import bgl
import blf
import Font
from TextInput import TextInput

inited=False

elements = []

class TextInputBLF(TextInput):
    def __init__(self, font, size=12, word_wrap=True):
        global elements, draw
        TextInput.__init__(self, 0, word_wrap)
        if isinstance(font, Font.Font):
            self.font = Font
        else:
            self.font = Font.Font(font, size)
        self.align = 2
        self.window_width=0
        self.window_height=0
        self.lines = 10
        
        self.draw_cursor = True
        
        self.x, self.y, self.w, self.h = 0,0,0,0
        
        elements.append(self)
        if not draw in logic.getCurrentScene().post_draw:
            logic.getCurrentScene().post_draw.append(draw)
            
        self.lines_cache = None
        self.lines_cache_count = None
        self.size_cache = self.font.size
    
    def set_pos(self, pos):
        if pos < 0: pos = 0
        if pos > len(self.data): pos = len(self.data)
        if pos != self.pos:
            self.lines_cache = None
            self.pos = pos
    
    def set_lines_count(self, n):
        if not self.lines_cache or n>len(self.lines_cache):
            self.lines_cache = None
            self.lines=n
            return
        if n<len(self.lines_cache):
            if self.align==0: self.lines_cache = self.lines_cache[-n:]
            elif self.align==2: self.lines_cache = self.lines_cache[:n]
            self.lines = n
    
    def set_align(self, n):
        if not n in (0, 2): raise Exception("todo")
        if n!=self.align:
            #self.lines_cache = None
            self.align = n
    
    def set_begin(self, begin):
        self.lines_cache = None
        TextInput.set_begin(self, begin)
        
    def set_data(self, data):
        self.lines_cache = None
        TextInput.set_data(self, data)
        
    def set_end(self, end):
        self.lines_cache = None
        TextInput.set_end(self, end)
    
    def kill(self):
        global elements, draw
        if self in elements:
            elements.remove(self)
        self.font = None

        if not elements and draw in logic.getCurrentScene().post_draw:
            logic.getCurrentScene().post_draw.remove(draw)

        
    def draw(self):
        if not self.font: return
        if self.align not in (0, 2): return #TODO:
        width, height = render.getWindowWidth(), render.getWindowHeight()
        if (self.size_cache != self.font.size or self.window_width != width) and self.align in (0, 2):
            self.lines_cache = None
            self.window_width = width
            self.size_cache = self.font.size
            self.resize( int( (width-2*2) / (self.font.getSize("Wq"*15)[0]/30) ) )

        if self.lines_cache == None:
            self.cursor = ""
            lines = list(self.gen_output())
            self.lines_cache_count = len(lines)
            lines = lines[-self.lines:]
            #if self.align == 2: lines.reverse()
            self.lines_cache = lines
        else: lines = self.lines_cache
        
        lh = self.font.getSize("Wq")[1]+3
        if self.align == 0:
            y = lh
        elif self.align == 2:
            y = height
            y = y - (lh - self.font.getSize("W")[1])
        
        for i in range(len(lines)):
            if self.align==2:
                i=len(lines)-i-1
            line = lines[i]
            self.font.draw(line, 2, y, valign=2, fix=False)
            if self.draw_cursor and not self.cursor and i==self.cursor_coords[0] - (self.lines_cache_count - len(lines)):
                bgl.glColor4f(*self.font.color)
                bgl.glBegin(bgl.GL_LINES)
                try:
                    x_pos= self.font.getSize("W"*self.cursor_coords[1])[0]
                    y_size = self.font.getSize("W")[1]
                    bgl.glVertex2i(x_pos+2, height-y-1)
                    bgl.glVertex2i(x_pos+2, height-(y-1-y_size))
                finally: bgl.glEnd()
                
            if self.align==0: y+=lh
            elif self.align==2: y-=lh

def init(cont):
    pass

def draw():
    global elements
    if not elements: return

    Font.setOrtho2D()
    Font.fixOffset()
    Font.preDraw()
    
    for elem in elements:
        elem.draw()

    Font.postDraw()