#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from json import JSONDecoder, JSONEncoder
from threading import RLock
from base64 import b64decode
import time
import traceback

class ConnExc(Exception): pass

class Connection:
    def __init__(self, sock):
        self.sock=sock
        self.io_lock=RLock()
        self.send_lock = RLock()
        self.closed=False
        self.jd=JSONDecoder()
        self.je=JSONEncoder(ensure_ascii=False)
        self.charset = 'utf-8'
        self.traffic = [0, 0]
        self.timeout = 30.0
        #self.sock.settimeout(self.timeout)
        self.last_time = time.time()
    
    def close(self):
        if self.closed: return
        #self.io_lock.acquire()
        try:
            try: self.sock.close()
            except: pass
            self.closed = True
        finally: pass#self.io_lock.release()
        
    def reader(self):
        buf = b""
        while 1:
            self.io_lock.acquire()
            try: data = self.sock.recv(4096)
            except:
                self.io_lock.release()
                if self.closed: return
                yield None
                continue
            else:
                self.io_lock.release()
            
            if len(data) == 0:
                if self.closed: return
                if time.time()-self.last_time > self.timeout: return
                continue
            self.last_time = time.time()
            self.traffic[0] += len(data)
            if len(buf) + len(data) > 65536: raise ConnExc("too long")
            buf += data
            if not b"\x00" in data:
                continue

            elems = buf.split(b"\x00")
            buf = elems.pop()
            
            for elem in elems:
                try:
                    j = self.jd.decode(elem.decode(self.charset, "replace"))
                    if j==None: continue
                except: print(repr(elem));raise ConnExc("not json")
                yield j
                
    def send(self, data, sendall = False):
        try:
            data = self.je.encode(data)
        except:
            raise ConnExc("not json")
        if not isinstance(data, bytes): data = data.encode(self.charset)
        self.send_lock.acquire()
        try:
            tm = self.sock.gettimeout()
            self.sock.settimeout(self.timeout)
            if sendall: self.sock.sendall(data + b"\x00")
            else: self.sock.send(data + b"\x00")
            self.sock.settimeout(tm)
            self.traffic[1] += len(data)+1
        except:
            self.close()
            raise ConnExc("closed")
        finally:
            self.send_lock.release()
        
        
    def __del__(self): self.close()

if __name__=="__main__":
    import time

    cache = [
        {"command":"hello", "version":"1"},
        {"command":"login","user":"andreymal","passwd":"1234"},
        {"command":"blendlist"},
        #{"command":"get", "action":"start", "name":'2013.03.31.16.24.29.507'},
        #{"command":"get", "seek":0, "action":"read", "name":'2013.03.31.16.24.29.507'},
        #{"command":"get", "seek":1*32*1024, "action":"read", "name":'2013.03.31.16.24.29.507'},
        #{"command":"get", "seek":2*32*1024, "action":"read", "name":'2013.03.31.16.24.29.507'},
        #{"command":"get", "seek":3*32*1024, "action":"read", "name":'2013.03.31.16.24.29.507'},
        #{"command":"get", "seek":4*32*1024, "action":"read", "name":'2013.03.31.16.24.29.507'},
        #{"command":"get", "action":"close", "name":'2013.03.31.16.24.29.507'}
        
    ]

    #fp = open("load.blend","wb")

    def reader(c):
        read = c.reader()
        try:
            while 1:
                d = next(read)  
                #if 'command' in d.keys() and d['command']=='get':
                #    if d['action']=='read':
                #        fp.write(b64decode(d['data'].encode("utf-8")))
                #        d['data'] = "BASE64"
                #    elif d['action']=='closed':
                #        fp.close()
                print(d)
                
        except StopIteration: print("stop")

    import socket
    import threading
    addr, port = "localhost", "26600"#input("Address: ").rsplit(" ",1)
    port = int(port)
    sock = socket.socket()
    sock.connect((addr, port))

    c = Connection(sock)
    threading.Thread(None, reader, None, (c,)).start()

    r = ""
    try:
        while r!="q":
            time.sleep(0.25)
            if cache:
                r = cache.pop(0)
                print("> "+str(r))
            else:
                r = input("> ")
            if r=="q": break
            if not r: continue
            if isinstance(r, (str)): r=c.jd.decode(r)
            c.send(r)  
    except KeyboardInterrupt: pass
    except EOFError: pass
